/*
############################################################################
# This program is used to test the basic functions of the SD file system.
# It implements simple versions of the cat, rm, ls, echo, cd, pwd, mkdir and
# rmdir commands plus the <, > and >> file redirection operators.
# The program starts up the file driver and then prompts for a command.
#
# Written by Dave Hein
# Copyright (c) 2011, 2012 Parallax, Inc.
# MIT Licensed
############################################################################
*/

extern "C" {
#include <stdio.h>
#include <propeller.h>
#include <sys/sd.h>
#include <errno.h>
}


#include "FFDS1.h"
//#include "FullDuplexSerialPlus"

// The loader will automatically initialize the pin settings for the SD card
// if the board CFG file contains the SD pin description, and the ELF file
// contains the symbol table (i.e., the -s option was not specified).
// Otherwise, the mount function can be called by defining CALL_MOUNT and one
// one of the board types listed below.  You can also specify your own SD pin
// configuration by editing one of the examples that most closely matches your
// specific board.
extern "C"{
extern _Driver _FileDriver;

/* This is a list of all drivers we can use in the
 * program. The default _InitIO function opens stdin,
 * stdout, and stderr based on the first driver in
 * the list (the serial driver, for us)
 */
_Driver *_driverlist[] = {
  &_FileDriver,
  NULL
};

FFDS1 debug;
//extern int errno ;

int32_t manual_mount()
{
    debug.Str("Load and mount SD: ");
    _SD_Params* mountParams = (_SD_Params*)-1;

// Important: This code assumes you're using a C3 card.
// If you're using different hardware, make sure you
// change the following initialization to match your card!

    static _SD_Params params =
    {
        AttachmentType: _SDA_SingleSPI,
        pins:
        {
            SingleSPI:
            {
                MISO: 10,   // The pin attached to the SD card's MISO or DO output
                CLK:  11,   // The pin attached to the SD card's CLK or SCLK input
                MOSI: 12,   // The pin attached to the SD card's MOSI or DI input
                CS:   13    // The pin attached to the SD card's CS input
            }
        }
    };
    mountParams = &params;

    if (mountParams == (_SD_Params*)-1)
    {
        debug.Str("You must specify the SD paramters in the filetest.c\n");
        return 1;
    }

    uint32_t mountErr = dfs_mount(mountParams);
    if (mountErr)
    {
        debug.Str("Mount error:");
        debug.Dec(mountErr);
        debug.Tx('\n');
        return 1;
    }

    debug.Str("done.\r\n");
}
}
/* The program starts the file system. */
int main()
{
    debug.Start(31, 30, 230400);
    debug.Str("SD Test.\n\r");
    debug.Str("Starting Errno: ");
    debug.Dec(errno);
    debug.Str("\r\n");

    manual_mount();

    debug.Str("After mount Errno: ");
    debug.Dec(errno);
    debug.Str("\r\n");

    char buf[80];
    char abe[] = "Everthing you read on the internet is true. - Abraham Lincoln\n";
    int  size = strlen(abe)+1;




    debug.Str("Writing to file.\r\n");
    FILE* fp = fopen("abe2.txt", "w");
    if(fp) {
        int32_t count = fwrite(abe, 1, size, fp);
        fclose(fp);

        debug.Str("Wrote ");
        debug.Dec(count);
        debug.Str(" number of elements.");
    }
    else
    {
        debug.Str("Couldn't write the file. Errno: ");
        debug.Dec(errno);
        debug.Str("\r\n");

    }

    debug.Str("Opening file. \r\n");
    fp = fopen("abe2.txt", "r");
    if(fp) {
        fread(buf, 1, 80, fp);
        debug.Str(buf);
        fclose(fp);
    }
    else
    {
        debug.Str("Couldn't read the file. Errno: ");
        debug.Dec(errno);
        debug.Str("\r\n");
    }

    return 0;
}

/*
+--------------------------------------------------------------------
|  TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+------------------------------------------------------------------
*/
